/*
 * Copyright J-Squared Technologies Inc.
 *
 * All rights reserved. No part of this software may be re-produced,
 * re-engineered, re-compiled, modified, used to create derivatives, stored in a
 * retrieval system, or transmitted in any form or by any means, electronic,
 * mechanical, photocopying, recording, or otherwise without the prior written
 * permission of J-Squared Technologies Inc.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "rfm2g_api.h"

// Select One RFM Memory Offset based on Size of Computer RAM to Create an 8MB DMA Buffer

//#define RFM_MEM_OFFSET 0xFF800000 // 4GB Memory (4096M - 8M = 4088M : use mem=4088M as kernel boot parameter)
//#define RFM_MEM_OFFSET 0x1FF800000 // 8GB Memory (8192M - 8M = 8184M : use mem=8184M as kernel boot parameter)
//#define RFM_MEM_OFFSET 0x3FF800000 // 16GB Memory (16384M - 8M = 16376M : use mem=16376M as kernel boot parameter)
//#define RFM_MEM_OFFSET 0x7FF800000 // 32GB Memory (32768M - 8M = 32760M : use mem=32760M as kernel boot parameter)
//#define RFM_MEM_OFFSET 0x7FFF80000 // 64GB Memory (65536M - 8M = 65528M : use mem=65528M as kernel boot parameter)
//#define RFM_MEM_OFFSET 0x17FF800000 // 96GB Memory (98304M - 8M = 98296M : use mem=98296M as kernel boot parameter)
//#define RFM_MEM_OFFSET 0x1FFF800000 // 128GB Memory (1310724M - 8M = 131064M : use mem=131064M as kernel boot parameter)

//Select One RFM Page Size based on Size of RFM Board

//#define RFM_PAGE_SIZE 0x8000 // 128MB RFM
//#define RFM_PAGE_SIZE 0x10000 // 256MB RFM

// Select One Test Data Size based on Size of RFM Board

//#define BUF_SIZE 0x8000000 // 128MB RFM
//#define BUF_SIZE 0x10000000 // 256MB RFM

volatile RFM2G_UINT8 *pbyte;		// Byte Write Pointer mapped to RFM area
RFM2G_UINT8 bytebuffer[BUF_SIZE];	// Byte Read Buffer

int main(int argc, char *argv[])
{
	RFM2G_STATUS result;			// Return codes from RFM2Get API calls
	RFM2GHANDLE Handle = NULL;		// RFM Driver Handle
	
	int count;
	
	struct timespec startread, endread, startwrite, endwrite;
	int status;
	
	// Open the Reflective Memory device
	result = RFM2gOpen("/dev/rfm2g0", &Handle);
	if (result != RFM2G_SUCCESS)
	{
		printf("ERROR: RFM2gOpen() failed.\n");
		printf("ERROR MSG: %s\n", RFM2gErrorMsg(result));
		return (-1);
	}

	// Set DMA Threshold to 1 to FORCE Use of DMA 
	result = RFM2gSetDMAThreshold(Handle, 1);
	if (result != RFM2G_SUCCESS)
	{
		printf("ERROR: RFM2gSetDMAThreshold() failed.\n");
		printf("ERROR MSG: %s\n", RFM2gErrorMsg(result));
		return (-1);
	}
    
	printf("\nDMA Testing Write and Read Bytes for Full RFM Map ...\n\n");

	// Run DMA Test with Full RFM Mapped as Bytes

	// Map Pointer
	result = RFM2gUserMemory(Handle, (volatile void **)(&pbyte), RFM_MEM_OFFSET | RFM2G_DMA_MMAP_OFFSET, RFM_PAGE_SIZE);
	if (result != RFM2G_SUCCESS)
	{
		printf("ERROR: RFM2gUserMemory() failed.\n");
		printf("ERROR MSG: %s\n", RFM2gErrorMsg(result));
		// Close the Reflective Memory device
		RFM2gClose(&Handle);
		return (-1);
	}
	
	//Run 255 Cycles with Changing Values
	for (count = 0; count < 256; count++)
	{
		memset((void *)pbyte, count, sizeof(bytebuffer));
		
		// Get Write Start Time
		clock_gettime(CLOCK_MONOTONIC, &startwrite);
		
		// Write the data to RFM Memory
		result = RFM2gWrite(Handle, 0, (void *)pbyte, sizeof(bytebuffer));
		
		// Get Write End Time
		clock_gettime(CLOCK_MONOTONIC, &endwrite);
		
		// Get Read Start Time
		clock_gettime(CLOCK_MONOTONIC, &startread);
		
		// Use read to collect the data
		result = RFM2gRead(Handle, 0, (void *)pbyte, sizeof(bytebuffer));
		
		// Get Read End Time
		clock_gettime(CLOCK_MONOTONIC, &endread);

		// Compare the data buffers
		memcpy(&bytebuffer, (void *)pbyte, sizeof(bytebuffer));
		status = memcmp((void *)pbyte, &bytebuffer, sizeof(bytebuffer));    
		if (status != 0)
		{
			printf("Compare Test Failed");

			// Close the Reflective Memory device
			RFM2gClose(&Handle);
			return (-1);
		}	
		
		printf("Test Value = %3d | DMA Write Elapsed Time = %ld Microseconds | DMA Read Elapsed Time = %ld Microseconds\n", count, ((endwrite.tv_sec - startwrite.tv_sec) * 1000000) + ((endwrite.tv_nsec - startwrite.tv_nsec) / 1000), ((endread.tv_sec - startread.tv_sec) * 1000000) + ((endread.tv_nsec - startread.tv_nsec) / 1000));
	}

	// Unmap Pointer
	result = RFM2gUnMapUserMemory(Handle, (volatile void **)(&pbyte), RFM_PAGE_SIZE);
	if (result != RFM2G_SUCCESS)
	{
		printf("ERROR: RFM2gUnMapUserMemory() failed.\n");
		printf("ERROR MSG: %s\n", RFM2gErrorMsg(result));

		// Close the Reflective Memory device
		RFM2gClose(&Handle);
		return (-1);
	}
    
	printf("\nTest Passed!\n\n");

    return (0);
}
